"""Top-level package for RP To-Do."""

__app_name__ = "nhp-prep"
__version__ = "0.1.1"

(
    SUCCESS,
    DIR_ERROR,
    FILE_ERROR,
    JSON_ERROR,
    ID_ERROR,
    YAML_ERROR,
) = range(6)

ERRORS = {
    DIR_ERROR: "Configuration directory error",
    FILE_ERROR: "Configuration file error",
    ID_ERROR: "ID error",
}



import os
import re
import shutil

import pandas as pd

from app.main_logger import logger
from datetime import datetime

EPOCH_PATTERN = r'\d{13}'
DIGITS_REGEX = r'\D'
EMPTY_SPACE = ''
SEPARATOR = '_'
COLON = ':'
COMMA = ','
DOT = '.'
DATE_ENDING_CHAR = 'h'
EXP_COL = 'Exp'
SUB_COL = 'Sub'
RESEARCHER_COL = 'Researcher'
NAME_COL = 'Name'
EXP_NAME_END = 'data.csv'
EXP_START_TS = 'ExpStartTimestamp'
STD_FILENAME_PATTERN = r'^\d{4}-\d{2}-\d{2}_\d{4}h_([a-zA-Z]+(?:_[a-zA-Z]+)*)_[a-zA-Z]+_[a-zA-Z]{2,6}_data\.csv$'


def find_epoch(file: str):
    """Method that finds the EPOCH
    timestamp within the file.

    Args:
        file (str): The file to evaluate.

    Returns:
        string: The digits of the epoch timestamp only.
    """
    _compiled = re.compile(EPOCH_PATTERN)
    file_elements_list = file.split(SEPARATOR)
    _epoch = EMPTY_SPACE
    try:
        _filtered = list(filter(_compiled.match, file_elements_list))
        _epoch = list(_filtered)[0]
        """Removes any non-digit character from the epoch
        """
        _epoch = re.sub(DIGITS_REGEX, EMPTY_SPACE, _epoch)
    except Exception:
        logger.error(
            f'Error detected in file: {file}. Please check the error: ', exc_info=True)
    return _epoch


def get_params_from_file(filepath: str):
    """Function that retrieves the list of the 
    parameters that combined will create the 
    new filename for the current file.

    Args:
        filepath (str): The entire file path.

    Returns:
        list: List with the following parameters:
            - Exp -> Experiment name
            - Sub -> Subject name
            - Researcher or Name -> The name of the researcher
    """
    params_list = list()
    try:
        file_df = pd.read_csv(filepath)
    except Exception:
        logger.error(
            f'Error found in the file: {filepath}. Please check for the issues: ', exc_info=True)
        return params_list

    for each in [EXP_COL, SUB_COL, RESEARCHER_COL]:
        try:
            if each in file_df.columns and pd.notnull(file_df[each].iat[0]):
                params_list.append(file_df[each].iat[0])
        except Exception:
            logger.error(
                'Error has occurred while trying to retrieve one of the column names. Please check: ', exc_info=True)
    if (RESEARCHER_COL not in file_df.columns or pd.isnull(file_df[RESEARCHER_COL].iat[0])) and NAME_COL in file_df.columns:
        if pd.isnull(file_df[NAME_COL].iat[0]): #dealing with the case when there is no researcher name
            params_list.append("AMBIGUOUS")
        else:
            params_list.append(file_df[NAME_COL].iat[0])
    return params_list


def find_std_pattern(filename: str):
    """Function to check if the filename 
    already matches the current standard pattern
    of the file.

    It uses a REGEXP to evaluate the match or not.

    Args:
        filename (str): The filename to analyze.

    Returns:
        bool: True if the filename already matches the pattern.
              False if the filename does not match.
    """
    _std_compiled = re.compile(STD_FILENAME_PATTERN)
    if _std_compiled.fullmatch(filename):
        logger.warning(f'The filename {filename} already matches the pattern.')
        logger.warning(f'Skipping this file...')
        return True
    return False


def copy_or_rename_file(filepath: str, new_filename: str, output=None):
    """Function to copy or rename the file based on the input.
    If the output path is not set, then this function will rename
    the file within its original path. Otherwise, it will copy the 
    original file to a new location and it will update the name
    to follow the current standard for naming.

    Args:
        filepath (str): The full filename of the file.
        new_filename (str): It is the new filename.
        output (str, optional): Output path. Defaults to None.

    Returns:
        new_full_name (str): The new fullpath name.
    """
    original_path, _file = os.path.split(filepath)
    new_full_filename = ''
    # Renaming/Copying section
    if output is not None:
        if not os.path.exists(output):
            os.mkdir(output)  # Make sure the output path exists
        new_full_filename = os.path.join(output, new_filename)
        try:
            shutil.copy(filepath, new_full_filename)
        except Exception:
            logger.error(
                f'Error found in the file: {filepath}. Please check for the issues: ', exc_info=True)
            return False
    else:
        new_full_filename = os.path.join(original_path, new_filename)
        os.rename(filepath, new_full_filename)
    return new_full_filename


def file_rename(filepath: str, output=None):
    """Function that renames a data file based on the 
    standard:

    YYYY-MM-DD_HHmmh_<experiment_name>_<Subject_name>_<Researcher_name_or_initials>_data.csv

    Args:
        filepath (str): The file to rename.
        output (str): The output path where the renamed 
                        file is going to be saved.

    Returns:
        boolean: True if the file was successfully renamed, 
                 False if the file cannot be renamed.
    """
    _, filename = os.path.split(filepath)

    # First evaluates if the file already matches the pattern or not.
    if find_std_pattern(filename=filename):
        # Here, we keep the file as is or copy to a new location.
        copy_or_rename_file(filepath=filepath,
                            new_filename=filename, output=output)
        return True

    """First, let's find the Epoch timestamp in the filepath.
    """
    epoch_timestamp = find_epoch(filename)
    if not epoch_timestamp.strip():
        logger.info(
            f'The current file {filename} does not have any timestamp or there is an error. Check in the logs!')
        return False

    epoch_int = int(epoch_timestamp)
    # divided by 1k to get the milliseconds.
    epoch_transformed = datetime.fromtimestamp(epoch_int / 1000.0)

    # Gets the time with milliseconds for ExpStartTimestamp column
    epoch_time = str(epoch_transformed.time())[:-3]

    epoch_time = epoch_time.replace(DOT, COMMA)

    logger.debug(
        f'Current timestamp transformed [HH:MM:SS.sss] ==> {epoch_time}')

    # Extracting the HHMM from the epoch time.
    hours_minutes = epoch_time[:-7].replace(COLON, EMPTY_SPACE)

    date_format_list = [str(epoch_transformed.date()),
                        hours_minutes + DATE_ENDING_CHAR]
    try:
        exp_date_time_format = SEPARATOR.join(
            date_format_list)  # YYYY-MM-DD_HHmmh
        logger.debug(f'Datetime timestamp list ==> {date_format_list}')
        logger.debug(f'Datetime transformed string ==> {exp_date_time_format}')
    except Exception:
        logger.error(
            f'Error found in the file: {filepath}. Please check for the issues: ', exc_info=True)

    format_name_parts = get_params_from_file(filepath)
    if len(format_name_parts) != 3:
        logger.warning(
            'Not enough parameters to change the filename...')
        logger.warning('Exiting now!')
        return False

    # Inserts at the beginning the datetime format
    format_name_parts.insert(0, exp_date_time_format)

    # Appends at the end the experiment ending data.csv
    format_name_parts.append(EXP_NAME_END)
    try:
        logger.debug(f'Format parts: {format_name_parts}')
        new_filename = SEPARATOR.join(format_name_parts)
    except Exception:
        logger.error(
            f'Error found in the file: {filepath}. Please check for the issues: ', exc_info=True)
        return False

    new_fullpath = copy_or_rename_file(filepath=filepath,
                                       new_filename=new_filename, output=output)

    # Injects the experiment timestamp start in the corresponding column in the new file.
    new_df = pd.read_csv(new_fullpath)
    new_df[EXP_START_TS] = new_df[EXP_START_TS].fillna(epoch_time)
    new_df.to_csv(new_fullpath, index=False)

    return True



import logging
import logging.config
import pkg_resources
import yaml


stream_config = pkg_resources.resource_stream(__name__, 'config/logs.yaml')
config = yaml.safe_load(stream_config.read())
logging.config.dictConfig(config=config)


logger = logging.getLogger(__name__)



import pandas as pd
import datetime
from datetime import datetime as dt

from app.main_logger import logger

TRIAL_START_TIME_COL = "TrialStartTimestamp"
EXP_START_TIME_COL = "ExpStartTimestamp"
TIME_COL = "Time"

def change_timestamp(filename: str):
    """Function that creats an estimated time from the start time epoch
    timestamp and the Time column (which records the end time in ms
    since the start time)
    It will add the value in the Time column to the value in the 
    ExpStartTimestamp column, and parse the format to be hh:mm:ss;ms
    Args:
        filename (str): The name of the file to be modified

    Returns:
        file_date: The modified file data
    """
    file_data = pd.read_csv(filename)
    #1. read from the time column and add that to the start time
    for index,trial in file_data.iterrows():
        if not pd.isnull(trial[TRIAL_START_TIME_COL]): #if the file already has "TrialStartTimestamp", no need to change it
            continue
        #2. read timestamp of the start of the exp from col (example: 11:47:57,444)
        if pd.isnull(trial[EXP_START_TIME_COL]): #if the file does not contain a start timestamp
            continue
        if pd.isnull(trial[TIME_COL]): #don't have info to check the time
            continue
        start_time = dt.strptime(trial[EXP_START_TIME_COL], "%X,%f")
        time_from_start = trial["Time"]
        estimate_time = start_time + datetime.timedelta(milliseconds=time_from_start)
        datestr = estimate_time.strftime("%X,%f")[:-3] #To trim the last three zeros
        file_data.at[index, TRIAL_START_TIME_COL] = datestr

    return file_data



import pandas as pd
import time

from app.main_logger import logger

REF_START_COL = "Start"
REF_END_COL = "End"
REF_DATE_COL = "Date"
REF_SUB_COL = "Sub"

TRIAL_START_COL = "TrialStartTimestamp"
SUB_COL = "Sub"
SUB_TEST = "TEST"
SUB_AMBIGUOUS = "AMBIGUOUS"

HHMMSS_FORMAT = '%H:%M:%S'
HOUR_ENDING = "h"
HYPHEN_CHAR = "-"
SEPARATOR_TAG = "_"
BACKSLASH = "/"
COLON = ":"
DOUBLE_ZERO = "00"


def change_sub(file_path: str, filename: str, ref_filename: str):
    """Function that changes the subject name to the subject name
    specified in the reference file within the specified time range
    If the name is not specified, put "UNDEFINED" as the subject name.
    Args:
        file_path (str): the full path to the file to be changed
        filename (str): the name of the file to be changed
        ref_filename (str): the name of the reference file

    Returns:
        file_data: the modified content of the file
    """
    ref_data = pd.read_csv(ref_filename)
    file_data = pd.read_csv(file_path)

    # 1. parse the file name for the month, day and year, and change it to this format mm/dd/yyyy
    full_time_str = filename.split(HOUR_ENDING)[0]
    date_str = full_time_str.split(SEPARATOR_TAG)[0]
    dates_array = date_str.split(HYPHEN_CHAR)
    year = int(dates_array[0])
    month = int(dates_array[1])
    day = int(dates_array[2])
    new_date_format = str(month) + BACKSLASH + \
        str(day) + BACKSLASH + str(year)
    # 2. parse the file start time in this format: hh:mm:00
    hour_str = full_time_str.split(SEPARATOR_TAG)[1]
    hours = hour_str[:-2]
    minutes = hour_str[2:]
    new_hours_format = hours + COLON + minutes + COLON + DOUBLE_ZERO
    file_start_time = time.strptime(new_hours_format, HHMMSS_FORMAT)
    # 3. search for the first row that contains the matching date and time in the baboons file
    for index_ref, trial_ref in ref_data.iterrows():
        # find the corresponding date
        if new_date_format != trial_ref[REF_DATE_COL]:
            continue
        # 3_1 parse the start time to a date
        subject_start_time = time.strptime(
            trial_ref[REF_START_COL], HHMMSS_FORMAT)
        # 3_2 parse the end time to a date
        subject_end_time = time.strptime(
            trial_ref[REF_END_COL], HHMMSS_FORMAT)
        if subject_end_time < file_start_time:  # means the file does not include the subject
            continue
        sub = trial_ref[REF_SUB_COL]
        # 4. loop through the data file
        for index, trial in file_data.iterrows():
            # skip the lines that don't have a start time
            if pd.isnull(trial[TRIAL_START_COL]):
                continue
            # 4_1 parse TrailStartTimestamp to a date
            start_time = time.strptime(
                trial[TRIAL_START_COL], '%H:%M:%S,%f')
            # 4_2 test if the trial start time stamp is greater than the start time and less than the end time
            if start_time >= subject_start_time and start_time < subject_end_time:
                # 4_3 if yes, then change the subject to the corresponding subject in the baboons file
                file_data.at[index, SUB_COL] = sub

    # 5 change the subjects without a name to "AMBIGUOUS"
    for index, trial in file_data.iterrows():
        if trial[SUB_COL] == SUB_TEST:
            file_data.at[index, SUB_COL] = SUB_AMBIGUOUS
    return file_data



import pandas as pd

from app.main_logger import logger


def map_columns(columns: list, filename: str):
    """Function generated to reorder the columns
    of a CSV file.
    It will fill with empty spaces all of those 
    columns that do not exist in the file.
    Args:
        columns (list): The list of column names as reference.
        filename (str): The file to reorder

    Returns:
        dataframe: The dataframe version of the file according 
                    to the columns order.
    """
    ordered_df = pd.DataFrame()
    try:
        file_df = pd.read_csv(filename)
        ordered_df = file_df.reindex(columns=columns)
    except Exception as e:
        logger.error(
            f'An exception has been detected while processing the file: {filename}. Please check the trace: {e}')
    return ordered_df



from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()
with open("requirements.txt", "r", encoding="utf-8") as fh:
    requirements = fh.read()

setup(
    name='nhp-prep',
    version='0.1.1',
    author='Hugo Angulo, Zijun Zhao',
    author_email='hugoanda@andrew.cmu.edu, zijunzha@andrew.cmu.edu',
    license='MIT',
    description='Pre-processing data tool for NHP Lab @ CMU',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url='https://caoslab.psy.cmu.edu:32443/monkeylab/preprocessing-scripts',
    py_modules=['nhp-prep', 'app'],
    packages=find_packages(),
    install_requires=[requirements],
    keywords=['nhp-prep', 'caoslab', 'cmu', 'pre-processing'],
    python_requires='>=3.8',
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3.8",
    ],
    entry_points='''
        [console_scripts]
        nhp-prep=app.__main__:main
    ''',
    package_data={
        'config': ["*"]
    }
)



import click
import os

import pandas as pd

from app import __app_name__, __version__
from app.main_logger import logger
from app.column_mapper import map_columns
from app.subject_changer import change_sub
from app.timestamp_estimator import change_timestamp
from app.rename_files import file_rename
from time import time


FILE_TYPE = '.csv'
FILE_COLUMNS_STD = 'app/config/columns__std_format.csv'


def print_version(ctx, param, value):
    if not value or ctx.resilient_parsing:
        return
    click.echo(f'CLI: {__app_name__}')
    click.echo(f'Version: v{__version__}')
    ctx.exit()


def log_issues(issues_list: list):
    """Helper function to include in the logs
    the list of some of the issues found.

    Args:
        issues_list (list): List with empty or errors.
    """
    for err in issues_list:
        logger.error(err)


@click.group()
@click.option('--version', '-v', is_flag=True, callback=print_version,
              expose_value=False, is_eager=True, help='Shows current version.')
def main():
    logger.info(f'CLI: {__app_name__}')
    logger.info(f'Version: v{__version__}')


@click.command(name='reorder-columns', short_help='Reorders columns in files.')
@click.option('--input', '-i', type=click.Path(exists=True, file_okay=True), required=True, help='The input input with the files to process OR a solo CSV file.')
@click.option('--output', '-o', type=click.Path(exists=False), required=True, help='The output input for the reordered files.')
@click.option('--ref-file', '-r', type=str, help='Reference file that contains the names of the columns.')
def reorder_columns(input, output, ref_file):
    """
    Processes the files from an input directory or file, and
    returns the same file(s) to a different output
    directory. If the reference file is not included
    the application will retrieve an internal standard file
    that will serve as a reference.

    OUTPUT: 
    If the output path does not exist, it will be created.

    """
    logger.info(
        f'-------------------------- REORDERING COLUMNS --------------------------')
    logger.info(f'Input: {input}')
    logger.info(f'Output path: {output}')
    logger.info(f'Columns reference file: {ref_file}')

    reference_file = ref_file if ref_file is not None else FILE_COLUMNS_STD
    df_reference = pd.read_csv(reference_file)
    ref_columns = list(df_reference.columns)
    """Creates the output path in case that it does not
    exists.
    """
    if not os.path.exists(output):
        os.makedirs(output)

    _files_to_process = 0
    _error_files = 0
    # TODO: Needs a validation for a directory or a file.
    for _, _, files in os.walk(input):
        for each in files:
            if each.endswith(FILE_TYPE):
                _files_to_process += 1
                filename = os.path.join(input, each)
                out_filename = os.path.join(output, each)
                try:
                    df_reordered = map_columns(
                        columns=ref_columns, filename=filename)
                    df_reordered.to_csv(out_filename, index=False)
                    logger.info(f'File reordered successfully: {out_filename}')
                except Exception as e:
                    logger.error(
                        f'An error has been detected while processing the file: {filename}. \n Please check the following error: {e}')
                    _error_files += 1
    _reordered_files = _files_to_process - _error_files
    _percentage_success = (_reordered_files / _files_to_process) * 100
    logger.info(
        f'Total files reordered successfully:  {_reordered_files}/{_files_to_process} => {_percentage_success} % success!')


@click.command(name='rename', short_help='Renames all the files based on standard.')
@click.option('--input', '-i', type=click.Path(exists=True, file_okay=True), required=True, help='The input directory with the files to process OR a solo CSV file.')
@click.option('--output', '-o', type=click.Path(exists=False), required=False, help='The output path for the reordered file(s) [OPTIONAL].')
def rename(input, output):
    """
    Processes the file(s) from an input, and
    returns the same file(s) to a different output
    input, but with the format:

    YYYY-MM-DD_HHmmh_<experiment_name>_<Subject_name>_<Researcher_name_or_initials>_data.csv

    OUTPUT: 
    If the output path does not exist, it will be created.
    Also, if the Output path is not passed, nhp-prep will
    proceed to use the path of each file and rename the original
    file.
    """       
    start_time = time()
    logger.info(
        f'-------------------------- RENAMING FILE(S) --------------------------')
    logger.info(f'Input: {input}')
    logger.info(f'Output path: {output}')
    if not os.path.exists(output):
        os.makedirs(output)
    if os.path.isfile(input):
        logger.info('File recognized. \n Proceeding to rename it...')
        _result = file_rename(filepath=input, output=output)
        if _result is False:
            logger.error(
                f'An error occurred during the process of renaming the file: {input}. Please check the logs.')
        logger.info(f'Total execution time: {time() - start_time} seconds')
    elif os.path.isdir(input):
        logger.info('Path recognized.')
        logger.info('Proceeding to rename all files within it...')
        num_files = 0
        num_errors = 0
        error_files = list()
        empty_files = list()
        for each in os.listdir(input):
            if each.endswith(FILE_TYPE):  # Filtering only .csv files
                filepath = os.path.join(input, each)
                num_files += 1
                _result = file_rename(filepath=filepath, output=output)
                if _result is False:
                    logger.error(
                        f'An error occurred during the process of renaming the file: {each}. Please check the logs.')
                    try:
                        file_df = pd.read_csv(filepath_or_buffer=filepath)
                        if file_df.empty:
                            logger.error(
                                f'The file {each} has been detected to be empty. It can be ignored!')
                            empty_files.append(filepath)
                        else:
                            error_files.append(filepath)
                    except Exception:
                        logger.error(
                            f'The file {each} cannot be processed. Please check the logs: ', exc_info=True)
                        empty_files.append(filepath)
                    num_errors += 1

        _renamed_files = num_files - num_errors
        _percentage_success = (_renamed_files / num_files) * 100
        logger.error(f'The list of files with errors is:')
        log_issues(error_files)

        logger.error(f'The list of empty files is:')
        log_issues(empty_files)

        logger.info(f'Total files with errors: {len(error_files)}')
        logger.info(f'Total empty files: {len(empty_files)}')

        logger.info(
            f'Total files reordered successfully:  {_renamed_files}/{num_files} => {_percentage_success} % success!')
        logger.info(f'Total execution time: {time() - start_time} seconds')


@click.command(name='timestamp-estimate', short_help='Estimates trial true timestamp.')
@click.option('--input', '-i', type=str, help='Directory that contains all the CSV files to process.')
@click.option('--output', '-o', type=str, help='Output directory of the processed files.')
def timestamp_estimate(input, output):
    """This command estimates the timestamp of an specific trial
    inside of each file(s). In order for this command to work
    the file must have the following columns:

      - Time \n
      - ExpStartTimestamp
      - TrialStartTimestamp

    """
    logger.info(
        f'-------------------------- ESTIMATING TIMESTAMP IN output(S) --------------------------')
    logger.info(f'Input directory: {input}')
    logger.info(f'Output directory: {output}')
    """Creates the output path in case that it does not
    exists.
    """
    if not os.path.exists(output):
        os.makedirs(output)

    _files_to_process = 0
    _error_files = 0
    for _, _, files in os.walk(input):
        for each in files:
            if each.endswith(FILE_TYPE):
                _files_to_process += 1
                filename = os.path.join(input, each)
                out_filename = os.path.join(output, each)
                try:
                    df_timestamp_changed = change_timestamp(
                        filename = filename
                    )
                    df_timestamp_changed.to_csv(out_filename, index=False)
                    logger.info(f'Change timestamp successfully: {out_filename}')
                except Exception as e:
                    logger.error(
                        f'An error has been detected while processing the file: {filename}. \n Please check the following error: {e}')
                    _error_files += 1
    _reordered_files = _files_to_process - _error_files
    _percentage_success = (_reordered_files / _files_to_process) * 100
    logger.info(
        f'Total files reordered successfully:  {_reordered_files}/{_files_to_process} => {_percentage_success} % success!')


@click.command(name='sub-rename', short_help='Updates the subject name based trial level.')
@click.option('--ref-file', '-f', required=True, help='Reference file that contains the names of subjects and timestamp start and end.')
@click.option('--input', '-i', required=True, help='Input directory.')
@click.option('--output', '-o', help='Output directory.')
def sub_rename(input, output, ref_file):
    logger.info(
        f'-------------------------- SUBJECT RENAMING --------------------------')
    logger.info(f'Input directory: {input}')
    logger.info(f'Output directory: {output}')
    logger.info(f'Reference file: {ref_file}')
    """
    Processes the files from a directory, and
    returns the same files to a different output
    directory. If the reference file is not included
    the application will retrieve a standard file
    that will serve as a reference.

    DIRECTORY: The input directory with the files to process.

    OUTPUT: The output directory for the reordered files.
    If the output path does not exist, it will be created.

    REF-FILE: The file that contains the date, and the start
    and end timestamp of each subjects.
    """

    logger.info(f'Input directory: {input}')
    logger.info(f'Output directory: {output}')
    logger.info(f'Columns reference file: {ref_file}')

    reference_file = ref_file if ref_file is not None else './app/config/baboons_sessions_compiled.csv.csv'
    """Creates the output path in case that it does not
    exists.
    """
    if not os.path.exists(output):
        os.makedirs(output)

    _files_to_process = 0
    _error_files = 0
    for _, _, files in os.walk(input):
        for each in files:
            if each.endswith(FILE_TYPE):
                _files_to_process += 1
                filename = os.path.join(input, each)
                out_filename = os.path.join(output, each)
                try:
                    df_changed = change_sub(
                        file_path=filename, filename=each, ref_filename=reference_file)
                    df_changed.to_csv(out_filename, index=False)
                    logger.info(
                        f'Subject changed successfully: {out_filename}')
                except Exception as e:
                    logger.error(
                        f'An error has been detected while processing the file: {filename}. \n Please check the following error: {e}')
                    _error_files += 1
    _changed_files = _files_to_process - _error_files
    _percentage_success = (_changed_files / _files_to_process) * 100
    logger.info(
        f'Total files reordered successfully:  {_changed_files}/{_files_to_process} => {_percentage_success} % success!')


main.add_command(reorder_columns)
main.add_command(rename)
main.add_command(timestamp_estimate)
main.add_command(sub_rename)

if __name__ == '__main__':
    main()
